# Package for managing mailing lists
#
# $Id: List.pm,v 1.49.4.3 2000/04/21 16:39:33 bmula Exp $
#
# Written by Brion Moss (brion@cobaltnet.com, brion@nda.com)
#            Kevin K.M. Chiu (kevin@cobaltnet.com)
#	     Will DeHaan (will@cobalnet.com)
# Unwritten and adapted for majordomo MLM by Sridhar Gopal (gsri@cobaltnet.com)
#
# Copyright 1998 Cobalt Networks http://www.cobaltnet.com/
#

package Cobalt::List;
use AutoLoader 'AUTOLOAD';
BEGIN {
require Exporter;
@ISA    = qw(Exporter);
@EXPORT = qw(
	     alias_get_bytarget
	     alias_exist alias_get alias_get_vacationless
             alias_set alias_add alias_delete
	     list_exist list_create list_delete list_modify
	     list_is_mem list_get_mem list_set_mem
	     list_add_mem list_del_mem
	     list_lists list_lists_mem list_lists_del_mem
	     list_sites_lists list_sites_is_mem list_sites_del_mem
	     list_sites_create list_sites_delete list_sites_modify
	     list_config_open list_config_closed
	     );

require Cobalt::Utility;
import Cobalt::Utility qw(:DEFAULT);
require Cobalt::Email;
import Cobalt::Email qw(:DEFAULT);
require Cobalt::Netutil;
import Cobalt::Netutil qw(netutil_get_fulldomain);
require Cobalt::Vsite;
import Cobalt::Vsite qw(vsite_get_bygroup);
}

use File::Copy;
use integer;
use strict;
1;

__END__

sub alias_get_bytarget
# Returns all the aliases which go to a given target
#   In other words, if the aliases file looks like:
#     ron.decline: ron
#     rdecline: ron
#   then alias_get_bytarget("ron") would return ("ron.decline","rdecline");
# Arguments: recipient address
# Return value: an array of ( alias1, alias2, ... )
# Note: group aliases all go into the majordomo aliases file, so we don't
#   need to filter those out.  The only alias we could collide with would
#   be forwarding aliases.
{
    my ($target) = @_;
    my @list;
    "joe: $target" =~ /^(.+):\s*$target\s*$/;  # precompile the regexp
    open (ALIAS, "$Aliasfile") || return ("4999 list_cant_read_aliases");
    while ( <ALIAS> ) {
      if (//) {
	next if (Cobalt::User::user_exist($1));  # skip forwarding aliases
	push(@list,$1);
      }
    }
    close ALIAS;
    return @list;
}

sub alias_exist
# Does the given alias exist?
# Arguments: alias name
# Return value: true if it does, false if it doesn't
{
    my ($alias, $group) = @_;
    my $majordomo_alias = $alias . '_' . $group;
    my $vsite_name = (vsite_get_bygroup($group))[1];
    my $email_alias = "$alias\@$vsite_name";
    return 1 if (Cobalt::List::email_alias_exist_in_file($email_alias, $EmailVirtUser));
    return 1 if (Cobalt::List::alias_exist_in_file($alias, $Aliasfile));
    return Cobalt::List::alias_exist_in_file($majordomo_alias, $Majordomo_aliases);
}

sub email_alias_exist_in_file
# Does the given alias exist in this file?
# Arguments: alias name, file name
# Return value: true if it does, false if it doesn't
{
    my ($alias, $file) = @_;
    return 0 if (! -e $file);
    open(ALIAS, $file) ||
	return("4999 list_cant_read_alias_file");
    while (<ALIAS>) {
	if (/^s*(\S+)\s*(.*)$/o) {
	    if  ($alias eq $1) {
		close ALIAS;
		return 1;
	    }
	}
    }
    close ALIAS;
    return 0;
}

sub alias_exist_in_file
# Does the given alias exist in this file?
# Arguments: alias name, file name
# Return value: true if it does, false if it doesn't
{
    my ($alias, $file) = @_;
    return 0 if (! -e $file);
    open(ALIAS, $file) ||
	return("4999 list_cant_read_alias_file");
    "$alias:" =~ /^$alias:/; # compile once per function call
    while (<ALIAS>) {
	(close ALIAS, return 1) if (//);
    }
    close ALIAS;
    return 0;
}

sub alias_get
# Returns all the aliases for an address
# Arguments: address
# Return value: an array of ( member1, member2... )
{
    my ($user) = @_;
    my @list = Cobalt::List::alias_get_from_file($user, $Aliasfile);
    return $list[0] if ($list[0] =~ /^\d{4} \w/);
    return @list unless ($#list < 0);
    return Cobalt::List::alias_get_from_file($user, $Majordomo_aliases);
}

sub alias_get_vacationless
# Returns all the aliases for an user but for vacation and \$user
# Arguments: alias name
# Return value: an array of ( member1, member2... )
{
    my ($user) = @_;
    my @aliases = Cobalt::List::alias_get_from_file($user, $Aliasfile);
    return $aliases[0] if ($aliases[0] =~ /^\d{4} \w/);
    return @aliases unless grep /$Vaca_prog/o, @aliases;
    @aliases = grep !/$Vaca_prog/o, @aliases;
    return grep !/^\\$user$/, @aliases;
}

sub alias_get_from_file
# Returns all the aliases for an address in a file
# Arguments: address, file name
# Return value: an array of ( member1, member2... )
{
    my ($user, $file) = @_;
    my @list;
    open( ALIAS, "< $file") || return ("4999 list_cant_read_alias_file");
    while ( <ALIAS> ) {
	@list = split(/,\s*/,$1) if (/^$user:\s*(.+)$/);
    }
    close ALIAS;
    return @list;
}

sub alias_set
# Update aliases of the given user with the provided list
# Arguments: user name, new array of aliases
# Return value: number of actual aliases set
{
    my ($user, @aliases) = @_;
    return "4999 list_invalid_adding_self" if (grep(/^$user$/,@aliases));
    my $lock;
    return $lock if (($lock = Cobalt::List::lock_aliases()) !~ /^2/);
    open(ALIAS, $Aliasfile) ||
	Cobalt::List::abort("4999 list_cant_read_aliases");
    open( ALIAS_NEW, "> $Lockdir/aliases") ||
	Cobalt::List::abort("4999 list_cant_write_aliases");
    "$user:" =~ /^$user:/; # compile once per function call
    while (<ALIAS>) {
	print ALIAS_NEW unless (//);
    }
    close ALIAS;
    print ALIAS_NEW "$user:\t" . join(", ", Cobalt::List::uniq(sort @aliases)) . "\n"
	if ($#aliases >= 0);
    close ALIAS_NEW;
    rename ("$Lockdir/aliases", $Altmp) ||
	Cobalt::List::abort("4999 list_cant_rename_aliases");
    return Cobalt::List::abort($lock) if (($lock = Cobalt::List::commit_aliases()) !~ /^2/);
    return("2999 list_alias_set_ok");
}

sub alias_add
# Add new entries to an existing alias; if no such alias exists, add a new one
# Arguments: user name, new array of aliases
# Return value: number of actual aliases added
{
    my ($user, @aliases) = @_;
    my @oldaliases;
    if (scalar (@oldaliases = Cobalt::List::alias_get($user))) {
	return Cobalt::List::alias_set($user, @oldaliases, @aliases);
    }
    else {
	return Cobalt::List::alias_set($user, @aliases);
    }
}

sub alias_delete
# Remove the specified aliases from the aliasfile
# Arguments: array of aliases to be removed
# Return value: number of actual aliases removed
{
    my (@aliases) = @_;
    my ($lock, $line, $count);
    return("2999 list_alias_delete_ok") if ($#aliases < 0);
    return $lock if (($lock = Cobalt::List::lock_aliases()) !~ /^2/);
    open(ALIAS, $Aliasfile) ||
	Cobalt::List::abort("4999 list_cant_read_aliases");
    open( ALIAS_NEW, "> $Lockdir/aliases") ||
	Cobalt::List::abort("4999 list_cant_write_aliases");
    while ($line = <ALIAS>) {
	my ($alias) = split(/:/, $line);
	print ALIAS_NEW $line unless grep /$alias/, @aliases;
	++$count;
    }
    close ALIAS;
    close ALIAS_NEW;
    rename ("$Lockdir/aliases", $Altmp) ||
	Cobalt::List::abort("4999 list_cant_rename_aliases");
    return Cobalt::List::abort($lock) if (($lock = Cobalt::List::commit_aliases()) !~ /^2/);
    return("4999 list_alias_invalid") unless $count;
    return("2999 list_alias_delete_ok");
}

######################################################

sub list_exist
# Does the given list exist?
# Arguments: list name
# Return value: true if it does, false if it doesn't
{
    my ($list) = @_;
    $list =~ tr[A-Z][a-z];	# lowercase the characters
    return (-e "$Majordomo_dir/lists/$list");
}

sub list_create
# Create a list with the specified membership
# Arguments: array of list name, members
# Return value: number of members actually added, 0 otherwise
# Side effects: add/udpate file(s) belonging to the list
{
    my ($list, @members) = @_;
    my $lock;
    $list =~ tr[A-Z][a-z];	# lowercase the characters
    return ("4999 list_invalid_already_exists")
	if (list_exist($list));
#    return "4999 list_invalid_adding_self" if (grep(/^$list$/,@members));
    return $lock if (($lock = Cobalt::List::lock_aliases()) !~ /^2/);
    open( LIST, "> $Majordomo_dir/lists/$list" ) ||
	Cobalt::List::abort("4999 list_cant_write_list");
    print LIST join "\n", Cobalt::List::uniq(sort @members);
    if ($#members < 0) {
	print LIST "nobody\n";
    }
    else {
	print LIST "\n";
    }
    close LIST;
    chmod 0664, "$Majordomo_dir/lists/$list";
    chown (scalar getpwnam($Majordomo_user), scalar getgrnam($Majordomo_group),
        "$Majordomo_dir/lists/$list");
    open( ALIAS, ">> $Majordomo_aliases") ||
	Cobalt::List::abort("4999 list_cant_append_maj_aliases");
    print ALIAS <<EOF;
$list:\t"|$Majordomo_dir/wrapper resend -l $list $list-list"
$list-list:\t:include:$Majordomo_dir/lists/$list
owner-$list:\t$Majordomo_list_owner
$list-request:\t"|$Majordomo_dir/wrapper request-answer $list"
$list-approval:\t$Majordomo_list_owner

EOF
    close ALIAS;
    unlock_aliases();

    # cannot send mail during installation or sysreset
    # kludge..never do it -- gsri
#    if (-e "$Majordomo_aliases") {
#       open (MAIL, "| $Mail_cmd majordomo") ||
#           return ("4999 list_cant_send_mail");
#       print MAIL "\nconfig $list $list.gooey\n";
#       close MAIL;
#    }
    return("2999 list_create_ok");
}

sub list_delete
# Delete the specified lists
# Arguments: lists to be deleted
# Return value: number of lists actually deleted
# Side effects: remove files belonging to the lists
{
    my (@lists, $lock);
    while ($_ = shift) {
        # cannot send mail during installation or sysreset
        # kludge..never do it -- gsri
#       next unless -f "$Majordomo_dir/lists/$_.config";
	push @lists, $_;
	tr[A-Z][a-z];		# lowercase the characters
	unlink "$Majordomo_dir/lists/$_.intro";
	unlink "$Majordomo_dir/lists/$_.info";
	unlink "$Majordomo_dir/lists/$_.auto";
	unlink "$Majordomo_dir/lists/$_.open";
	unlink "$Majordomo_dir/lists/$_.closed";
	unlink "$Majordomo_dir/lists/$_.config";
    }
    return $lock if (($lock = Cobalt::List::lock_aliases()) !~ /^2/);
    open( ALIAS, "$Majordomo_aliases") ||
	Cobalt::List::abort("4999 list_cant_read_maj_aliases");
    open( ALIAS_NEW, "> $Lockdir/aliases.majordomo") ||
	Cobalt::List::abort("4999 list_cant_write_maj_aliases");
    my $line;
    while ($line = <ALIAS>) {
	if ($line =~ /^owner-(\S+):/ || $line =~ /^(\S+)-list:/ ||
	    $line =~ /^(\S+)-request:/ || $line =~ /^(\S+)-approval:/ ||
	    $line =~ /^(\S+):/) {
	    print ALIAS_NEW $line unless (grep /^$1$/, @lists);
	}
    }
    close ALIAS_NEW;
    close ALIAS;
    unlink map "$Majordomo_dir/lists/$_", @lists;
    move("$Lockdir/aliases.majordomo", $Majordomo_aliases) or
	abort("4999 list_cant_move_maj_aliases");
    unlock_aliases();
    return("2999 list_delete_ok");
}

sub list_modify
# Modify the specified list
# Arguments: old listname, new listname, new members
# Return value: usual error string
# Side effects: if old listname != new listname, we could either nix the
#     old list and create a new one or update only the config file (not
#     the info, intro and other paraphrenalia). the rationale: one
#     could delete and recreate the list if they want to do the
#     former. we will do the latter though it is more work.
#     -- gsri 1999/06/18 09:47:01 EDT
{
    my ($oldlist, $list, @members) = @_;
    $list =~ tr[A-Z][a-z];	# lowercase the characters
    $oldlist =~ tr[A-Z][a-z];	# lowercase the characters

    if ($oldlist ne $list) {
	# return error of new listname exists already
	return ("4999 list_invalid_already_exists")
	    if (list_exist($list));

	# rename oldlist files into list files.
	opendir DIR, "$Majordomo_dir/lists" or
	    return("4999 list_cant_open_listdir");
	"${oldlist}.intro"
	    =~ /^${oldlist}(\.(intro|info|auto|open|closed|config))?$/;
	map {
	    move("$Majordomo_dir/lists/$_",
	        "$Majordomo_dir/lists/$list$1") if (//);
	} readdir DIR;
	closedir DIR;

	# obtain the lock
	my $lock;
	return $lock if (($lock = lock_aliases()) !~ /^2/);

	# update config file, if any
	if (open CONFIG, "$Majordomo_dir/lists/$list.config") {
	    open TMPCONF, "$Lockdir/$list.config";
	    while (<CONFIG>) {
		print TMPCONF s/\b$oldlist\b/$list/g;
	    }
	    close TMPCONF;
	    close CONFIG;
	    move("$Lockdir/$list.config", "$Majordomo_dir/lists/$list.config") or
		abort("4999 list_cant_move_config");
	    chmod 0664, "$Majordomo_dir/lists/$list.config";
	    chown (scalar getpwnam($Majordomo_user),
		   scalar getgrnam($Majordomo_group),
		   "$Majordomo_dir/lists/$list.config");
	}
	
	# update aliases file
	open( ALIAS, "$Majordomo_aliases") or
	  abort("3999 list_cant_read_maj_aliases");
	open( ALIAS_NEW, "> $Lockdir/aliases.majordomo") or
	  abort("3999 list_cant_write_maj_aliases");

	"${oldlist}:" =~ /^($oldlist|${oldlist}-list|owner-${oldlist}|${oldlist}-request|${oldlist}-approval):/;
	while (<ALIAS>) {
	    print ALIAS_NEW unless //;
	}
	print ALIAS_NEW <<EOF;
$list:\t"|$Majordomo_dir/wrapper resend -l $list $list-list"
$list-list:\t:include:$Majordomo_dir/lists/$list
owner-$list:\t$Majordomo_list_owner
$list-request:\t"|$Majordomo_dir/wrapper request-answer $list"
$list-approval:\t$Majordomo_list_owner

EOF
        close ALIAS_NEW;
	close ALIAS;
	move("$Lockdir/aliases.majordomo", $Majordomo_aliases) or
	    abort("3999 list_cant_move_maj_aliases");

	# unlock
        unlock_aliases();
    }

    # update members of the list
    my $ret;
    return $ret unless ($ret = list_set_mem($list, @members));
    return("2999 list_modify_ok");
}

######################################################

sub list_is_mem
# Checks for membership in a list
# Arguments: list name, member name
# Return value: true if member is in list, false otherwise
{
    my ($list, $user) = @_;
    return 0 if ($user eq "nobody");
    $list =~ tr[A-Z][a-z];	# lowercase the characters
    open( LIST, "$Majordomo_dir/lists/$list") ||
	return ("4999 list_cant_read_list");
    while ( <LIST> ) {
	chop;
	(close LIST, return 1) if $user eq $_;
    }
    close LIST;
    return 0;
}

sub list_get_mem
# Returns the membership of a list
# Arguments: list name
# Return value: an array of ( member1, member2... )
{
    my ($list) = @_;
    my @members;
    $list =~ tr[A-Z][a-z];	# lowercase the characters
    open( LIST, "$Majordomo_dir/lists/$list") ||
	return ("4999 list_cant_read_list");
    while ( <LIST> ) {
	chop;
	next if (/^nobody$/);
	push @members, $_;
    }
    close LIST;
    return @members;
}

sub list_set_mem
# (Re)sets the membership of a list
# Arguments: list name
# Return value: number of members if successful, 0 if not
# Side effects: modify list membership file
{
    my ($list, @members) = @_;
    $list =~ tr[A-Z][a-z];	# lowercase the characters
    return "4999 list_invalid_adding_self" if (grep(/^$list$/,@members));
    my $lock;
    return $lock if (($lock = Cobalt::List::lock_aliases()) !~ /^2/);
    open( LIST, "> $Lockdir/$list.majordomo" ) ||
	Cobalt::List::abort("4999 list_cant_write_newlist");
    print LIST join "\n", Cobalt::List::uniq(sort @members);
    if ($#members < 0) {
	print LIST "nobody\n";
    }
    else {
	print LIST "\n";
    }
    close LIST;
    move("$Lockdir/$list.majordomo", "$Majordomo_dir/lists/$list") or
	abort("4999 list_cant_move_list");
    chmod 0664, "$Majordomo_dir/lists/$list";
    chown (scalar getpwnam($Majordomo_user), scalar getgrnam($Majordomo_group),
        "$Majordomo_dir/lists/$list");
    Cobalt::List::unlock_aliases();
    return("2999 list_set_membership_ok");
}

######################################################

sub list_add_mem
# Adds members to a list
# Arguments: list name, new members
# Return value: actual number of members added
{
    my ($list, @new_mem) = @_;
    my (@mem, $return);
    return $mem[0] if ((@mem = list_get_mem($list))[0] =~ /^\d{4} \w/);
    return $return
	if (($return = list_set_mem($list,@mem,@new_mem)) !~ /^2/);
    return("2999 list_add_membership_ok");
}

sub list_del_mem
# Removes members from a list
# Arguments: list name, old members
# Return value: actual number of members removed
{
    my ($list, @old_members) = @_;
    my (@members, @new_members, $member, $return);
    return $members[0] if ((@members = list_get_mem($list))[0] =~ /^\d{4} \w/);
    while ($member = shift @members) {
	 push @new_members, $member unless (grep {$member eq $_} @old_members);
    }
    return $return
	unless ($return = Cobalt::List::list_set_mem($list, @new_members));
    return("2999 list_delete_members_ok");
}

######################################################

sub list_lists
# Returns a list of all the mailing lists
# Arguments: none
# Return value: an array of all the mailing lists
{
    opendir (DIR, "$Majordomo_dir/lists") ||
	return ("4999 list_cant_open_listdir");

    my (@lists, $list);
    foreach $list (sort readdir (DIR)) {
        next if ($list =~ /[^-_0-9a-z]/);       # skip non-list files
        next if ($list =~ /^(RCS|CVS|core)$/);  # files and directories to ignore
        next if (-d "$Majordomo_dir/lists/$list"); # skip directories
	push @lists, $list;
    }

    closedir (DIR);
    return @lists;
}

sub list_lists_mem
# Returns a list of all the lists and their members in a 2D array
# Arguments: none
# Return value: an array of ( list name, member1, member2, ... )
{
    my @lists;
    return $lists[0] if ((@lists = list_lists())[0] =~ /^\d{4} \w/);
    return map {
	my $list=$_;
	my @mem;
	return $mem[0] if ((@mem = list_get_mem($list))[0] =~ /^\d{4} \w/);
        [$list, @mem];
	} sort @lists;
}

sub list_lists_del_mem
# Delete the specified users from all lists
# Arguments: none
# Return value: number of actual deletions
{
    # i bow to thee oh, Crom! gimme the power to destroy these evil @users!
    my (@users) = @_;
    my @lists;
    return $lists[0] if ((@lists = list_lists())[0] =~ /^\d{4} \w/);
    # drop the results to the floor and move on!
    map Cobalt::List::list_del_mem($_, @users), @lists;
    return("2999 list_delete_all_ok");
}

######################################################

sub list_sites_lists
# Returns a list of all mailing lists associated with a site
# Arguments: site group name
# Return value: an array of all the mailing lists
{
    my ($group) = @_;
    my (@lists, $list, @out);
    # If we get an error back from list_lists(), just return it
    return $lists[0] if ((@lists = list_lists())[0] =~ /^\d{4} \w/);
    "a_$group" =~ /^(.+)_$group$/; # precompile the regexp
    foreach $list (@lists) {
	push @out, $1 if ($list =~ //);
    }
    return @out;
}

sub list_sites_is_mem
# Check whether a user is a member of any site
# Arguments: user name
# Return value: the site user is registered with, if any; 0 otherwise
{
    my ($user) = @_;
    my ($list, @lists);
    return $lists[0] if ((@lists = list_lists())[0] =~ /^\d{4} \w/);
    foreach $list (@lists) {
	next unless ($list =~ /^(.+)-users$/);
	return $1 if (Cobalt::List::list_is_mem($list, $user) == 1);
    }
    return 0;
}

sub list_sites_del_mem
# Delete the user from a site list, if any
# Arguments: none
# Return value: the site user was registered with, if any; 0 otherwise
{
    my ($user) = @_;
    my ($list, $group, @lists);
    return $lists[0] if ((@lists = list_lists())[0] =~ /^\d{4} \w/);
    foreach $list (@lists) {
	next unless ($list =~ /^(.+)-users$/);
	$group = $1;
	# absolutely unnecessary to check for membership!...
	# just because list_del_mem cannot truly say whether it found
        # this user or not ... list processing should be much easier
        # than the way it is setup in sauce  - gsri@rant.com 11/01
	next unless (Cobalt::List::list_is_mem($list, $user) == 1);
	Cobalt::List::list_del_mem($list, $user);
	return $group;
    }
    return 0;
}

sub list_sites_create
# Create a list for a particular site
# Arguments: site group name, listname, members
# Return value: true if successful, false otherwise
{
    my ($group, $list, @members) = @_;
    my $ret;
    $list =~ tr[A-Z][a-z];	# lowercase the characters
    return $ret if (($ret = list_create("${list}_${group}", @members)) !~ /^2/);

    # i don't really like this code here -- gsri
    # add entries to virtusertable so $list@$fqdn works
    my $fqdn = ($group eq $MainGroup) ?
	netutil_get_fulldomain() : (vsite_get_bygroup($group))[1];
    my %alias = ("${list}\@${fqdn}" => "${list}_${group}",
		 "${list}-list\@${fqdn}" => "${list}_${group}-list",
		 "${list}-approval\@${fqdn}" => "${list}_${group}-approval",
		 "${list}-request\@${fqdn}" => "${list}_${group}-request",
		 "owner-${list}\@${fqdn}" => "owner-${list}_${group}");
    return $ret if (($ret = mail_virtuser_add_alias(%alias)) !~ /^2/);

    return ("2999 list_create_ok");
}

sub list_sites_delete
# Delete lists for a particular site
# Arguments: site group name, array of lists
# Return value: true if successful, false otherwise
# Side effects: remove files belonging to the lists
{
    my ($group, @lists) = @_;
    my $ret;
    map tr[A-Z][a-z], @lists;	# lowercase the characters
    return $ret if (($ret = list_delete(map "${_}_${group}", @lists)) !~ /^2/);

    # i don't really like this code here -- gsri
    # delete entries from virtusertable
    my $list;
    my $fqdn = ($group eq $MainGroup) ?
	netutil_get_fulldomain() : (vsite_get_bygroup($group))[1];
    foreach $list (@lists) {
	my %alias = ("${list}\@${fqdn}" => "${list}_${group}",
		     "${list}-list\@${fqdn}" => "${list}_${group}-list",
		     "${list}-approval\@${fqdn}" => "${list}_${group}-approval",
		     "${list}-request\@${fqdn}" => "${list}_${group}-request",
		     "owner-${list}\@${fqdn}" => "owner-${list}_${group}");
	return $ret if (($ret = mail_virtuser_del_alias(%alias)) !~ /^2/);
    }

    return ("2999 list_delete_ok");
}

sub list_sites_modify
# Modify a list for a particular site
# Arguments: site group name, old listname, new listname, new members
# Return value: usual error string
# Side effects: read side effects of list_modify
{
    my ($group, $oldlist, $list, @members) = @_;
    my $ret;
    $list =~ tr[A-Z][a-z];	# lowercase the characters
    $oldlist =~ tr[A-Z][a-z];	# lowercase the characters
    return $ret if (($ret = list_modify("${oldlist}_${group}",
					"${list}_${group}", @members)) !~ /^2/);

    # i don't really like this code here -- gsri
    # modify entries in virtusertable
    my $fqdn = ($group eq $MainGroup) ?
	netutil_get_fulldomain() : (vsite_get_bygroup($group))[1];
    my(%aliases) = mail_virtuser_list_alias();
    my(%routes) = mail_virtuser_list_route();

    delete $aliases{"${oldlist}\@${fqdn}"};
    delete $aliases{"${oldlist}-list\@${fqdn}"};
    delete $aliases{"${oldlist}-approval\@${fqdn}"};
    delete $aliases{"${oldlist}-request\@${fqdn}"};
    delete $aliases{"owner-${oldlist}\@${fqdn}"};
    $aliases{"${list}\@${fqdn}"} = "${list}_${group}";
    $aliases{"${list}-list\@${fqdn}"} = "${list}_${group}-list";
    $aliases{"${list}-approval\@${fqdn}"} = "${list}_${group}-approval";
    $aliases{"${list}-request\@${fqdn}"} = "${list}_${group}-request";
    $aliases{"owner-${list}\@${fqdn}"} = "owner-${list}_${group}";
    mail_build_virtusertable(\%aliases, \%routes) or
	return "4999 list_cant_modify_aliases";

    return ("2999 list_modify_ok");
}

######################################################

sub list_config_open
# Open up lists for subscription
# Arguments: lists to be opened
# Return value: number of lists actually opened
{
    my (@lists) = @_;
    map {
	tr[A-Z][a-z];		# lowercase the characters
	my $return = Cobalt::List::list_config_update($_, "subscribe_policy", "open");
	return $return if ($return !~ /^2/);
    } @lists;
    return ("2999 list_config_open_ok");
}

sub list_config_closed
# Close lists for subscription
# Arguments: lists to be closed
# Return value: number of lists actually closed
{
    my (@lists) = @_;
    map {
	tr[A-Z][a-z];		# lowercase the characters
	my $return = Cobalt::List::list_config_update($_, "subscribe_policy", "closed");
	return $return if ($return !~ /^2/);
    } @lists;
    return ("2999 list_config_closed_ok");
}

sub list_config_update
# Update the config info for a list
# Arguments: list name, config variable, config value
# Return value: 1 on success, 0 means failure
{
    my ($list, $var, $val) = @_;
    my $lock;
    return $lock if (($lock = Cobalt::List::lock_aliases()) !~ /^2/);
    $list =~ tr[A-Z][a-z];	# lowercase the characters
    open (LIST, "< $Majordomo_dir/lists/$list.config") ||
	Cobalt::List::abort("4999 list_cant_read_config");
    open (LIST_NEW, "> $Lockdir/$list.config") ||
	Cobalt::List::abort("4999 list_cant_write_config");
    while (<LIST>) {
	if (/^$var/o) {
	    print LIST_NEW "$var = $val\n";
	}
	else {
	    print LIST_NEW;
	}
    }
    close LIST;
    close LIST_NEW;
    move("$Lockdir/$list.config", "$Majordomo_dir/lists/$list.config") or
	abort("4999 list_cant_move_config");
    chmod 0664, "$Majordomo_dir/lists/$list.config";
    chown (scalar getpwnam($Majordomo_user), scalar getgrnam($Majordomo_group),
        "$Majordomo_dir/lists/$list.config");
    Cobalt::List::unlock_aliases();
    return ("2999 list_config_update_ok");
}

######################################################

sub lock_aliases
# Lock the aliases file so that nobody can collide with us.
# Arguments: none
# Side effects: copies /etc/aliases to /etc/altmp
{
    if (-e $Altmp)
    {
	return ("4261 The aliases file is locked.");
    }
    else
    {
	# Lock the aliases file by creating the altmp file as a
	# copy of the aliases file.
	print STDERR "Copying aliases file $Aliasfile to $Altmp\n" if ($Debug);
	Cobalt::Utility::sauce_cp($Aliasfile,$Altmp);
    }
    print STDERR "Locked $Aliasfile with $Altmp\n" if ($Debug);

    open(LOCK,">$Lockdir/list-$$")
      || Cobalt::List::abort("4266 Cannot get aliases file lock: $!");
    print LOCK "$$\n$0";
    close LOCK;

    return "2260 Aliases file has been locked";
}

sub commit_aliases
# Commits the changes to the aliases file
# Side effects: renames $Altmp to $Aliasfile
# We keep a backup copy (aliases.old) in case newaliases barfs
{
  rename($Altmp,$Aliasfile)
    || return "4262 Cannot create new aliases file: $!";
  chmod(0644,$Aliasfile);
  unlink("$Lockdir/list-$$");
  return "2261 Aliases file changes have been committed";
}

sub unlock_aliases
# Unlocks the aliases file by wiping altmp
# Arguments: none
# Side effects: wipes altmp
{
    unlink($Altmp) || return "4264 Cannot delete altmp $Altmp: $!";
    print STDERR "Removed $Altmp\n" if ($Debug);
    unlink("$Lockdir/list-$$");
    return "2262 The aliases file is now unlocked";
}

sub abort
# leave in a graceful manner
# Arguments: a string to print
# Side effects: unlocks anything we've locked
{
    Cobalt::List::unlock_aliases()  if (-e "$Lockdir/list-$$");
    return join('',@_);
}

sub uniq
# Removes replicates from a sorted list
# Arguments: input list
# Return value: uniq-ed list
{
    my ($prev, @list);
    while ($_ = shift) {
	 push @list, ($prev = $_) if ($prev ne $_);
    }
    return @list;
}

1;
